package com.supervision.police.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.common.domain.R;
import com.supervision.police.domain.CasePerson;
import com.supervision.police.domain.ModelCase;
import com.supervision.police.dto.ModelCaseBase;
import com.supervision.police.dto.ModelCaseDTO;
import com.supervision.police.service.ModelCaseService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

@RestController
@Slf4j
@RequestMapping("/modelCase")
@RequiredArgsConstructor
public class ModelCaseController {

    private final ModelCaseService modelCaseService;

    /**
     * 查询案件列表
     * @param modelCase
     * @param page
     * @param size
     * @return
     */
    @Operation(summary = "查询案件列表")
    @PostMapping("/queryList")
    public R<IPage<ModelCaseDTO>> queryList(@RequestBody ModelCase modelCase,
                          @RequestParam(required = false, defaultValue = "1") Integer page,
                          @RequestParam(required = false, defaultValue = "20") Integer size) {
        IPage<ModelCaseDTO> modelCaseDTOIPage = modelCaseService.queryList(modelCase, page, size);
        return R.ok(modelCaseDTOIPage);
    }

    /**
     * 检查案件编号是否已存在
     * @param caseNo
     * @return
     */
    @GetMapping("/checkCaseNo")
    public R<?> checkCaseNo(@RequestParam @Parameter(name = "caseNo",description = "案件编号") String caseNo,
                            @RequestParam(required = false) @Parameter(name = "caseId",description = "案件id") String caseId) {
        return modelCaseService.checkCaseNo(caseNo,caseId);
    }

    /**
     * 新增或修改
     * @param modelCaseBase
     * @return
     */
    @PostMapping("/addOrUpd")
    public R<?> addOrUpd(@RequestBody ModelCaseBase modelCaseBase) {
        return modelCaseService.addOrUpd(modelCaseBase);
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @PostMapping("/del")
    public R<?> del(@RequestParam String id) {
        return modelCaseService.del(id);
    }

    /**
     * 查询案件人员信息列表
     * @param name
     * @return
     */
    @GetMapping("/getPerson")
    @Operation(summary = "查询案件人员信息列表")
    public R<List<CasePerson>> getPerson(@RequestParam String caseId,
                                         @RequestParam(required = false, defaultValue = "") String name) {
        return modelCaseService.getPerson(caseId, name);
    }

    @PostMapping("/addPerson")
    public R<?> addPerson(@RequestBody CasePerson person) {
        return modelCaseService.addPerson(person);
    }

    @PostMapping("/updatePerson")
    public R<Boolean> updatePerson(@RequestBody CasePerson person) {
        boolean success = modelCaseService.updatePerson(person);
        return R.ok(success);
    }

    /**
     * 导入案件信息
     * @param file
     * @return
     */
    @PostMapping("/uploadCase")
    public R<?> uploadCase(@RequestPart MultipartFile file) {
        return modelCaseService.uploadCase(file);
    }

    /**
     * 获取指标结果
     * @param caseId
     * @param indexType
     * @return
     */
    @PostMapping("/getIndexDetail")
    public R<?> getIndexDetail(@RequestParam String caseId,
                               @RequestParam String indexType,
                               @RequestParam(required = false, defaultValue = "1") Integer page,
                               @RequestParam(required = false, defaultValue = "20") Integer size) {
        return modelCaseService.getIndexDetail(caseId, indexType, page, size);
    }

}
