package com.supervision.police.controller;

import com.supervision.common.domain.R;
import com.supervision.police.dto.AnalyseCaseDTO;
import com.supervision.police.dto.CaseStatus;
import com.supervision.police.dto.caseScore.CaseScoreDetailDTO;
import com.supervision.police.service.ModelService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("model")
@RequiredArgsConstructor
public class ModelController {

    private final ModelService modelService;

    /**
     * 分析指标
     * @param analyseCaseDTO
     * @return
     */
    @PostMapping("/analyseCase")
    @Operation(summary = "分析指标")
    public R<?> analyseCase(@RequestBody AnalyseCaseDTO analyseCaseDTO) {
        return modelService.analyseCase(analyseCaseDTO);
    }


    /**
     * 获取指标详情
     * @param caseId 案件id
     * @return
     */
    @GetMapping("/caseScoreDetail")
    @Operation(summary = "案件得分详情功能")
    public R<CaseScoreDetailDTO> caseScoreDetail( @RequestParam @Parameter(name = "caseId",description = "案件id") String caseId) {

        CaseScoreDetailDTO detail = modelService.caseScoreDetail(caseId);
        return R.ok(detail);
    }

    @GetMapping("/exportCaseScoreDetail")
    @Operation(summary = "导出案件得分详情功能")
    public void exportCaseScoreDetail( @RequestParam @Parameter(name = "caseId",description = "案件id") String caseId,
                                       HttpServletResponse response) {

        modelService.exportCaseScoreDetail(caseId,response);
    }

    @GetMapping("/getCaseDateStatus")
    @Operation(summary = "获取案件数据状态")
    public R<CaseStatus> getCaseDateStatus(@RequestParam @Parameter(name = "caseId",description = "案件id") String caseId) {

        CaseStatus caseStatus = modelService.getCaseDateStatus(caseId);
        return R.ok(caseStatus);
    }

}
