package com.supervision.police.controller;

import com.supervision.common.domain.R;
import com.supervision.police.domain.ModelAtomicIndex;
import com.supervision.police.domain.ModelIndex;
import com.supervision.police.dto.CaseAtomicIndexDTO;
import com.supervision.police.dto.CaseAtomicResultWrapper;
import com.supervision.police.service.ModelIndexService;
import com.supervision.police.vo.ModelIndexReqVO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 指标表(ModelIndex)表控制层
 *
 * @author qmy
 * @since 2024-07-05 09:20:10
 */
@RestController
@RequestMapping("/modelIndex")
@RequiredArgsConstructor
public class ModelIndexController {

    private final ModelIndexService modelIndexService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param modelIndex 查询实体
     * @return 所有数据
     */
    @PostMapping("/selectAll")
    public R<?> selectAll(@RequestBody ModelIndexReqVO modelIndex,
                          @RequestParam(required = false, defaultValue = "1") Integer page,
                          @RequestParam(required = false, defaultValue = "20") Integer size) {
        return modelIndexService.selectAll(modelIndex, page, size);
    }

    /**
     * 新增/修改数据
     *
     * @param modelIndex 实体对象
     * @return 新增结果
     */
    @PostMapping("/addOrUpd")
    public R<?> addOrUpd(@RequestBody ModelIndex modelIndex) {
        return modelIndexService.addOrUpd(modelIndex);
    }

    /**
     * 删除数据
     *
     * @param id 主键
     * @return 删除结果
     */
    @PostMapping("/del")
    public R<?> del(@RequestParam String id) {
        return modelIndexService.del(id);
    }

    /*****************************原子指标**************/

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param modelAtomicIndex 查询实体
     * @return 所有数据
     */
    @PostMapping("/selectAllAtomic")
    public R<?> selectAllAtomic(@RequestBody ModelAtomicIndex modelAtomicIndex,
                                @RequestParam(required = false, defaultValue = "1") Integer page,
                                @RequestParam(required = false, defaultValue = "20") Integer size) {
        return modelIndexService.selectAllAtomic(modelAtomicIndex, page, size);
    }

    /**
     * 新增/修改数据
     *
     * @param modelAtomicIndex 实体对象
     * @return 新增结果
     */
    @PostMapping("/addOrUpdAtomic")
    public R<?> addOrUpdAtomic(@RequestBody ModelAtomicIndex modelAtomicIndex) {
        return modelIndexService.addOrUpdAtomic(modelAtomicIndex);
    }

    /**
     * 删除数据
     *
     * @param id 主键
     * @return 删除结果
     */
    @PostMapping("/delAtomic")
    public R<?> delAtomic(@RequestParam String id) {
        return modelIndexService.delAtomic(id);
    }


    /**
     * 查询案件原子指标列表
     * @return
     */
    @Operation(summary = "查询案件原子指标列表")
    @GetMapping("/listCaseAtomicIndex")
    public R<List<CaseAtomicIndexDTO>> pageListCaseAtomicIndex(
            @RequestParam @Parameter(name = "caseId",description = "案件id") String caseId,
            @RequestParam(defaultValue = "1")
            @Parameter(name = "indexSource",description = "指标来源 1:人工定义 2:数据库查询 3:图谱生成 4:大模型") String indexSource) {

        List<CaseAtomicIndexDTO> list = modelIndexService.listCaseAtomicIndex(caseId, indexSource);
        return R.ok(list);
    }


    /**
     * 保存案件原子指标评估结果
     * @param caseAtomicResultWrapper
     * @return
     */
    @Operation(summary = "保存案件原子指标评估结果")
    @PostMapping("/saveCaseAtomicResult")
    public R<Boolean> saveCaseAtomicResult(@RequestBody CaseAtomicResultWrapper caseAtomicResultWrapper) {

        Boolean result =  modelIndexService.saveCaseAtomicResult(caseAtomicResultWrapper);
        return R.ok(result);
    }
}

