package com.supervision.police.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.common.domain.R;
import com.supervision.police.dto.RoleDTO;
import com.supervision.police.service.SystemRoleService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

@Tag(name = "用户角色管理")
@RestController
@RequestMapping("/role")
@RequiredArgsConstructor
public class RoleController {

    private final SystemRoleService roleService;
    @Operation(summary = "新增角色")
    @PostMapping("/save")
    public R<String> saveRole(@RequestBody RoleDTO roleDTO) {
        String roleId = roleService.saveRole(roleDTO);
        return R.ok(roleId);
    }

    @Operation(summary = "修改角色")
    @PostMapping("/update")
    public R<?> updateRole(@RequestBody RoleDTO roleDTO) {
        roleService.updateRole(roleDTO);
        return R.ok();
    }

    @Operation(summary = "删除角色")
    @DeleteMapping("/delete")
    public R<Boolean> deleteRole(@Parameter(name = "id") @RequestParam(name="id") String id) {
        Boolean success = roleService.deleteRole(id);
        return R.ok(success);
    }
    @Operation(summary = "查看角色列表")
    @GetMapping("/list")
    public R<IPage<RoleDTO>> list(@Parameter(name = "roleName",description = "角色名称") @RequestParam(required = false) String roleName,
                               @Parameter(name = "pageNum",description = "页码") @RequestParam(defaultValue = "1") Integer pageNum,
                               @Parameter(name = "pageSize",description = "每页大小") @RequestParam(defaultValue = "10") Integer pageSize) {
        IPage<RoleDTO> list = roleService.list(roleName, pageNum, pageSize);
        return R.ok(list);
    }
}
