package com.supervision.police.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.common.domain.R;
import com.supervision.police.dto.user.*;
import com.supervision.police.service.SystemUserService;
import com.supervision.utils.UserUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

@Tag(name = "用户管理")
@RestController
@RequestMapping("/user")
@RequiredArgsConstructor
public class UserController {

    private final SystemUserService userService;


    @Operation(summary = "登录")
    @PostMapping("/login")
    public R<LoginResVO> login(@RequestBody LoginReqVO reqVO) {
        LoginResVO login = userService.login(reqVO);
        return R.ok(login);
    }

    @Operation(summary = "查看账号信息")
    @GetMapping("/getCurrentUser")
    public R<UserInfoDTO> getCurrentUser() {
        UserInfoDTO currentUser = userService.getCurrentUser();
        return R.ok(currentUser);
    }


    @Operation(summary = "新增用户信息")
    @PostMapping("/saveUserInfo")
    public R<String> saveUserInfo(@RequestBody UserInfoVoDTO userInfoVoDTO) {
        String userId = userService.saveUserInfo(userInfoVoDTO.toUserInfoReqVo());
        return R.ok(userId);
    }

    @Operation(summary = "修改用户信息")
    @PostMapping("/update")
    public R<?> updateUser(@RequestBody UserInfoVoDTO userInfoVoDTO) {
        userService.updateUserInfo(userInfoVoDTO.toUserInfoReqVo());
        return R.ok();
    }

    @Operation(summary = "修改用户状态")
    @PostMapping("/updateUserStatus")
    public R<?> updateUserStatus(@RequestBody UserStatusReqVo userStatusReqVo) {
        userService.updateUserStatus(userStatusReqVo);
        return R.ok();
    }

    @Operation(summary = "删除用户信息")
    @DeleteMapping("/delete")
    public R<Boolean> deleteUser(@Parameter(name = "id") @RequestParam(name="id") String id) {
        Boolean success = userService.deleteUser(id);
        return R.ok(success);
    }
    @Operation(summary = "查看用户信息列表")
    @GetMapping("/list")
    public R<IPage<UserInfoVoDTO>> list(@Parameter(name = "userName",description = "用户名") @RequestParam(required = false) String userName,
                                   @Parameter(name = "roleId",description = "角色id") @RequestParam(required = false) String roleId,
                                   @Parameter(name = "roleName",description = "角色名") @RequestParam(required = false) String roleName,
                                   @Parameter(name = "pageNum",description = "页码") @RequestParam(defaultValue = "1") Integer pageNum,
                                   @Parameter(name = "pageSize",description = "每页大小") @RequestParam(defaultValue = "10") Integer pageSize) {

        IPage<UserInfoDTO> page = userService.list(userName, roleId, roleName, pageNum, pageSize);

        return R.ok(page.convert(UserInfoVoDTO::new));
    }


    @Operation(summary = "修改密码")
    @PostMapping("changePassWord")
    public R<Boolean> changePassWord(@RequestBody UserInfoReqVo userInfo) {
        boolean b = userService.changePassWord(userInfo.getAccount(), userInfo.getPassword());
        return R.ok(b);
    }



}
