package com.supervision.police.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.supervision.police.domain.CaseEvidence;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

@Data
public class CaseEvidenceDetailDTO {

    @Schema(description = "案件证据id")
    private String id;

    @Schema(description = "案件证据名称")
    private String evidenceName;

    @Schema(description = "案件证据类型")
    private String evidenceType;

    @Schema(description = "案件证据类型描述")
    private String evidenceTypeDesc;

    @Schema(description = "证据提供人")
    private String provider;

    @Schema(description = "甲方")
    private String partyA;

    @Schema(description = "乙方")
    private String partyB;

    @Schema(description = "案件证据文件信息列表")
    private List<EvidenceFileDTO> fileList;

    @Schema(description = "最近时间")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private LocalDateTime updateTime;

    public CaseEvidenceDetailDTO() {
    }

    public CaseEvidenceDetailDTO(CaseEvidence caseEvidence, List<EvidenceFileDTO> fileList) {
        if (Objects.nonNull(caseEvidence)){
            this.id = caseEvidence.getId();
            this.evidenceName = caseEvidence.getEvidenceName();
            this.evidenceType = caseEvidence.getEvidenceType();
            this.updateTime = caseEvidence.getUpdateTime();
            this.provider = caseEvidence.getProvider();
            this.partyA = caseEvidence.getPartyA();
            this.partyB = caseEvidence.getPartyB();
        }
        this.fileList = fileList;
    }
}
