package com.supervision.police.dto.caseScore;

import cn.hutool.core.util.StrUtil;
import com.supervision.police.domain.ModelAtomicIndex;
import com.supervision.police.domain.ModelAtomicResult;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.springframework.ai.chat.ChatClient;

import java.util.Objects;

/**
 * 原子指标结果类
 */
@Data
public class AtomicResult {

    @Schema(description = "原子指标id")
    private String atomicId;

    @Schema(description = "原子指标名称")
    private String atomicName;

    @Schema(description = "指标id")
    private String indexId;

    @Schema(description = "指标来源 1:人工定义 2:数据库查询 3:图谱生成 4:大模型")
    private String indexSource;

    @Schema(description = "原子指标结果 1:真实 2:虚构 3：存在 4：不存在 5：未知")
    private String judgeResult;


    public AtomicResult() {
    }

    public boolean resultIsTrue() {
        return StrUtil.equals("1",this.judgeResult) || StrUtil.equals("3",this.judgeResult);
    }

    public AtomicResult(String indexId,ModelAtomicIndex modelAtomicIndex, ModelAtomicResult atomicResult) {
        if (Objects.isNull(atomicResult) || Objects.isNull(modelAtomicIndex)){
            return;
        }
        this.atomicId = modelAtomicIndex.getId();
        this.indexId = indexId;
        this.atomicName = modelAtomicIndex.getName();
        this.indexSource = modelAtomicIndex.getIndexSource();
        this.judgeResult = atomicResult.getAtomicResult();
    }

    public String buildDesc(ChatClient chatClient) {
        // 人工定义
        if (StrUtil.equals("1",this.indexSource)){

            return null;
        }

        // 数据库查询
        if (StrUtil.equals("2",this.indexSource)){
            return null;
        }

        // 图谱生成
        if (StrUtil.equals("3",this.indexSource)){
            return null;
        }

        // 大模型
        if (StrUtil.equals("4",this.indexSource)){
            return null;
        }

        return null;
    }
}
