package com.supervision.police.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.police.domain.CasePerson;
import com.supervision.police.mapper.CasePersonMapper;
import com.supervision.police.service.CasePersonService;
import com.supervision.police.service.CaseStatusManageService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Slf4j
@Service
@RequiredArgsConstructor
public class CasePersonServiceImpl extends ServiceImpl<CasePersonMapper, CasePerson> implements CasePersonService {


    private final CaseStatusManageService caseStatusManageService;
    @Override
    public boolean saveCaseActor(String caseId, String caseActorName, String caseActorIdCard) {

        if (StrUtil.isEmpty(caseId) || StrUtil.isEmpty(caseActorName) || StrUtil.isEmpty(caseActorIdCard)){
            log.info("案件行为人信息不能为空");
            return false;
        }

        Long count = super.lambdaQuery().eq(CasePerson::getCaseId, caseId).eq(CasePerson::getIdCard, caseActorIdCard).count();
        if (count > 0){
            log.info("案件行为人:{},已经存在于案件:{}之中", caseActorIdCard, caseId);
            return false;
        }

        CasePerson casePerson = new CasePerson();
        casePerson.setCaseId(caseId);
        casePerson.setName(caseActorName);
        casePerson.setIdCard(caseActorIdCard);
        casePerson.setCaseActorFlag(1);
        // 角色设置为行为人
        casePerson.setRoleCode("1");
        boolean save = super.save(casePerson);

        if (save){
            caseStatusManageService.whenSaveCasePeople(caseId, casePerson);
        }
        return save;

    }
}
