package com.supervision.police.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.constant.DataStatus;
import com.supervision.police.domain.ModelAtomicIndex;
import com.supervision.police.domain.ModelIndex;
import com.supervision.police.dto.JudgeLogic;
import com.supervision.police.dto.ModelIndexDTO;
import com.supervision.police.mapper.ModelAtomicIndexMapper;
import com.supervision.police.service.ModelAtomicIndexService;
import com.supervision.police.service.ModelIndexService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class ModelAtomicIndexServiceImpl extends ServiceImpl<ModelAtomicIndexMapper, ModelAtomicIndex> implements ModelAtomicIndexService {

    @Lazy
    @Autowired
    private ModelIndexService modelIndexService;
    @Override
    public ModelAtomicIndexMapper getMapper() {
        return super.getBaseMapper();
    }

    @Override
    public IPage<ModelAtomicIndex> selectAll(IPage<ModelAtomicIndex> iPage, ModelAtomicIndex index) {
        return super.getBaseMapper().selectAll(iPage, index);
    }

    @Override
    public List<ModelAtomicIndex> selectByCaseType(String caseType) {
        return super.getBaseMapper().selectByCaseType(caseType);
    }

    @Override
    public List<ModelAtomicIndex> selectBatchIds(List<String> ids) {
        return super.getBaseMapper().selectBatchIds(ids);
    }

    @Override
    public List<ModelAtomicIndex> listCaseAtomicIndex(List<String> indexIdList, String caseType, String indexSource) {
        return super.getBaseMapper().listCaseAtomicIndex(indexIdList, caseType, indexSource);
    }

    @Override
    @Transactional(transactionManager = "dataSourceTransactionManager",rollbackFor = Exception.class)
    public void whenDeleteAtomicIndex(String caseType,String atomicIndexId) {
        if (StrUtil.isEmpty(atomicIndexId)){
            log.warn("whenDeleteAtomicIndex: atomicIndexId is null,停止后续操作..");
            return;
        }
        // 清理指标中的关联数据
        // 获取指标列表
        List<ModelIndex> modelIndexList = modelIndexService.list(new LambdaQueryWrapper<ModelIndex>()
                .eq(StrUtil.isNotEmpty(caseType),ModelIndex::getCaseType, caseType)
                .eq(ModelIndex::getDataStatus, DataStatus.AVAILABLE.getCode()));

        List<ModelIndexDTO> modelIndexDTOList = modelIndexList.stream().map(ModelIndexDTO::new)
                .filter(modelIndexDTO -> modelIndexDTO.logicContainAtomicIndex(atomicIndexId)).toList();

        if (CollUtil.isEmpty(modelIndexDTOList)){
            return;
        }

        modelIndexDTOList.forEach(modelIndexDTO -> {
            List<JudgeLogic> judgeLogicList = modelIndexDTO.judgeLogicExcludeAtomicIndex(atomicIndexId);
            modelIndexDTO.setJudgeLogic(modelIndexDTO.judgeLogicToJson(judgeLogicList));
            modelIndexService.lambdaUpdate().eq(ModelIndex::getId, modelIndexDTO.getId()).set(ModelIndex::getJudgeLogic, modelIndexDTO.getJudgeLogic()).update();
        });
    }
}
