package com.supervision.police.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.common.domain.R;
import com.supervision.config.BusinessException;
import com.supervision.neo4j.domain.CaseNode;
import com.supervision.neo4j.domain.Rel;
import com.supervision.neo4j.service.Neo4jService;
import com.supervision.police.domain.*;
import com.supervision.police.dto.TripleInfoDTO;
import com.supervision.police.dto.TypeDTO;
import com.supervision.police.mapper.ModelRecordTypeMapper;
import com.supervision.police.mapper.NoteRecordSplitMapper;
import com.supervision.police.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StopWatch;

import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class ModelRecordTypeServiceImpl extends ServiceImpl<ModelRecordTypeMapper, ModelRecordType> implements ModelRecordTypeService {

    private final ModelRecordTypeMapper modelRecordTypeMapper;

    private final NoteRecordSplitMapper noteRecordSplitMapper;

    private final NotePromptService notePromptService;

    private final TripleInfoService tripleInfoService;

    private final Neo4jService neo4jService;

    private final OllamaChatClient chatClient;


    private final NotePromptTypeRelService notePromptTypeRelService;

    private final CaseTaskRecordService caseTaskRecordService;
    @Autowired
    private NoteRecordSplitService noteRecordSplitService;
    @Autowired
    private RecordSplitProcessService recordSplitProcessService;
    @Autowired
    private ModelRecordTypeService modelRecordTypeService;


    @Override
    public List<TypeDTO> queryTypeListChoose() {
        List<ModelRecordType> list = modelRecordTypeMapper.selectByName(null);
        return list.stream().map(e -> new TypeDTO(e.getId(), e.getRecordType())).collect(Collectors.toList());
    }

    @Override
    public List<ModelRecordType> queryType(String name, Integer page, Integer size) {
        List<ModelRecordType> list = modelRecordTypeMapper.selectByName(name);
        Map<String, TypeDTO> modelRecordTypeMap = list.stream().map(e -> new TypeDTO(e.getId(), e.getRecordType())).collect(Collectors.toMap(TypeDTO::getId, Function.identity()));
        // 获取类型和三元组提示词的关系表
        List<NotePromptTypeRel> relList = notePromptTypeRelService.lambdaQuery().list();
        Map<String, List<NotePromptTypeRel>> typeIdMap = relList.stream().collect(Collectors.groupingBy(NotePromptTypeRel::getTypeId));
        Map<String, List<NotePromptTypeRel>> promptIdMap = relList.stream().collect(Collectors.groupingBy(NotePromptTypeRel::getPromptId));

        for (ModelRecordType modelRecordType : list) {
            //笔录内容
            List<NoteRecordSplit> noteRecords = noteRecordSplitMapper.selectByRecordType(modelRecordType.getRecordType());
            modelRecordType.setRecords(noteRecords);
            //提示词
            // 根据类型表获取所有的prompt
            List<NotePromptTypeRel> promptRelList = typeIdMap.get(modelRecordType.getId());
            if (CollUtil.isNotEmpty(promptRelList)) {
                Set<String> promptIdSet = promptRelList.stream().map(NotePromptTypeRel::getPromptId).collect(Collectors.toSet());
                List<NotePrompt> prompts = Optional.ofNullable(notePromptService.listByIds(promptIdSet)).orElse(new ArrayList<>());
                for (NotePrompt prompt : prompts) {
                    List<String> modelRecordTypes = new ArrayList<>();
                    List<NotePromptTypeRel> promptTypeRelList = promptIdMap.get(prompt.getId());
                    for (NotePromptTypeRel notePromptTypeRel : promptTypeRelList) {
                        TypeDTO existType = modelRecordTypeMap.get(notePromptTypeRel.getTypeId());
                        modelRecordTypes.add(existType.getId());
                    }
                    prompt.setTypeList(modelRecordTypes);
                    prompt.setTripleList(buildTripleInfo(prompt));
                }
                modelRecordType.setPrompts(prompts);
            }
        }
        return list;
    }

    private List<TripleInfoDTO> buildTripleInfo(NotePrompt notePrompt) {
        List<TripleInfoDTO> list = new ArrayList<>();
        TripleInfoDTO dto = new TripleInfoDTO();
        dto.setType("头节点");
        dto.setTemplateName(notePrompt.getStartEntityTemplate());
        dto.setValue(notePrompt.getStartEntityType());
        list.add(dto);
        TripleInfoDTO dto1 = new TripleInfoDTO();
        dto1.setType("关系");
        dto1.setTemplateName(notePrompt.getRelTemplate());
        dto1.setValue(notePrompt.getRelType());
        list.add(dto1);
        TripleInfoDTO dto2 = new TripleInfoDTO();
        dto2.setType("尾节点");
        dto2.setTemplateName(notePrompt.getEndEntityTemplate());
        dto2.setValue(notePrompt.getEndEntityType());
        list.add(dto2);
        return list;
    }

    @Override
    public ModelRecordType queryByName(String content) {
        Wrapper<ModelRecordType> wrapper = new QueryWrapper<ModelRecordType>().eq("record_type", content);
        return modelRecordTypeMapper.selectOne(wrapper);
    }

    @Override
    @Transactional(transactionManager = "dataSourceTransactionManager", rollbackFor = Exception.class)
    public R<?> saveType(ModelRecordType type) {
        String id = type.getId();
        int i = 0;
        if (StringUtils.isEmpty(id)) {
            //新增
            i = modelRecordTypeMapper.insert(type);
        } else {
            i = modelRecordTypeMapper.updateById(type);
        }
        if (i > 0) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    @Transactional(transactionManager = "dataSourceTransactionManager", rollbackFor = Exception.class)
    public R<?> addOrUpdPrompt(NotePrompt prompt) {
        List<String> typeList = prompt.getTypeList();
        if (CollUtil.isEmpty(typeList)) {
            throw new RuntimeException("类型信息不能为空");
        }
        int i = 0;
        boolean save;
        if (StringUtils.isEmpty(prompt.getId())) {
            // 新增的时候,校验是否已经存在相同的三元组关系,如果已经存在了相同的三元组关系,不允许添加
            checkHasSameTriple(prompt.getStartEntityType(), prompt.getRelType(), prompt.getEndEntityType(), null);
            save = notePromptService.save(prompt);
            // 新增prompt绑定的分类信息
            for (String typeId : typeList) {
                NotePromptTypeRel rel = new NotePromptTypeRel();
                rel.setPromptId(prompt.getId());
                rel.setTypeId(typeId);
                notePromptTypeRelService.save(rel);
            }
        } else {
            checkHasSameTriple(prompt.getStartEntityType(), prompt.getRelType(), prompt.getEndEntityType(), prompt.getId());
            save = notePromptService.updateById(prompt);
            // 更新prompt绑定的分类信息
            // 首先查询已经有的,如果都存在,就不变,如果数据库有,前端没有,就删除,如果前端有,数据库没有,就新增
            List<NotePromptTypeRel> existDatabaseRelList = notePromptTypeRelService.lambdaQuery().eq(NotePromptTypeRel::getPromptId, prompt.getId()).list();

            if (CollUtil.isNotEmpty(existDatabaseRelList)) {
                Set<String> existTypeList = existDatabaseRelList.stream().map(NotePromptTypeRel::getTypeId).collect(Collectors.toSet());
                Set<String> frontRelIdList = new HashSet<>(typeList);
                // 删除(数据库有,前端没有的)
                List<String> deleteIdList = existTypeList.stream().filter(id -> !frontRelIdList.contains(id)).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(deleteIdList)) {
                    notePromptTypeRelService.lambdaUpdate().in(NotePromptTypeRel::getTypeId, deleteIdList).eq(NotePromptTypeRel::getPromptId, prompt.getId()).remove();
                }
                // 新增(前端有数据库没有的)
                frontRelIdList.stream().filter(id -> !existTypeList.contains(id)).forEach(e -> {
                    NotePromptTypeRel rel = new NotePromptTypeRel();
                    rel.setPromptId(prompt.getId());
                    rel.setTypeId(e);
                    notePromptTypeRelService.save(rel);
                });
            } else {
                // 如果数据库里面没查到,直接新增,一般不会走这一步
                for (String typeId : typeList) {
                    NotePromptTypeRel rel = new NotePromptTypeRel();
                    rel.setPromptId(prompt.getId());
                    rel.setTypeId(typeId);
                    notePromptTypeRelService.save(rel);
                }
            }
        }


        // 更新类型字段
        List<TripleInfoDTO> tripleList = prompt.getTripleList();
        for (TripleInfoDTO dto : tripleList) {
            if ("头节点".equals(dto.getType())) {
                notePromptService.lambdaUpdate().set(NotePrompt::getStartEntityTemplate, dto.getTemplateName())
                        .set(NotePrompt::getStartEntityType, dto.getValue())
                        .eq(NotePrompt::getId, prompt.getId()).update();
            } else if ("关系".equals(dto.getType())) {
                notePromptService.lambdaUpdate().set(NotePrompt::getRelTemplate, dto.getTemplateName())
                        .set(NotePrompt::getRelType, dto.getValue())
                        .eq(NotePrompt::getId, prompt.getId()).update();
            } else if ("尾节点".equals(dto.getType())) {
                notePromptService.lambdaUpdate().set(NotePrompt::getEndEntityTemplate, dto.getTemplateName())
                        .set(NotePrompt::getEndEntityType, dto.getValue())
                        .eq(NotePrompt::getId, prompt.getId()).update();
            }
        }

        if (save) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    private void checkHasSameTriple(String startEntityType, String relType, String endEntityType, String promptId) {
        List<NotePrompt> list = notePromptService.lambdaQuery().eq(NotePrompt::getStartEntityType, startEntityType)
                .eq(NotePrompt::getRelType, relType).eq(NotePrompt::getEndEntityType, endEntityType).list();
        if (CollUtil.isNotEmpty(list)) {
            if (StrUtil.isBlank(promptId)) {
                throw new RuntimeException("该三元组关系已经存在,请勿重复添加");
            } else {
                // 校验list查出来的是不是和promptId相等,如果不想等,也报错
                if (!list.get(0).getId().equals(promptId)) {
                    throw new RuntimeException("该三元组关系已经存在,请勿重复添加");
                }
            }

        }
    }

    @Override
    @Transactional(transactionManager = "dataSourceTransactionManager", rollbackFor = Exception.class)
    public R<?> delPrompt(NotePrompt prompt) {
        String id = prompt.getId();
        boolean removeById = notePromptService.removeById(id);
        // 删除同时从所有分类里面都删除掉
        notePromptTypeRelService.lambdaUpdate().eq(NotePromptTypeRel::getPromptId, id).remove();
        if (removeById) {
            return R.ok("删除成功");
        } else {
            return R.fail("删除失败");
        }
    }

    /*
     * 异步提交的任务,不回滚
     */
    @Override
    @Transactional(transactionManager = "dataSourceTransactionManager", rollbackFor = Exception.class, noRollbackFor = BusinessException.class)
    public List<TripleInfo> getThreeInfo(String caseId, String name, String recordId) {
        if (StrUtil.isBlank(recordId)) {
            throw new RuntimeException("笔录ID不能为空");
        }
        // 这里查询任务是否完成,如果完成了,就给结果
        Optional<CaseTaskRecord> caseTaskRecordOpt = caseTaskRecordService.lambdaQuery().eq(CaseTaskRecord::getCaseId, caseId)
                .eq(CaseTaskRecord::getRecordId, recordId).oneOpt();
        if (caseTaskRecordOpt.isPresent()) {
            CaseTaskRecord caseTaskRecord = caseTaskRecordOpt.get();
            // 0未执行 1正在执行 2执行成功 3执行超时
            if (caseTaskRecord.getStatus() == 1) {
                throw new BusinessException("笔录解析任务未完成,请等待");
            }
            if (caseTaskRecord.getStatus() == 0 || caseTaskRecord.getStatus() == 3) {
                // 重新提交
                caseTaskRecordService.lambdaUpdate().set(CaseTaskRecord::getStatus, 1).eq(CaseTaskRecord::getId, caseTaskRecord.getId()).update();
                List<ModelRecordType> allTypeList = modelRecordTypeService.lambdaQuery().list();
                // 根据recordId查询所有的分割后的笔录
                List<NoteRecordSplit> list = noteRecordSplitService.lambdaQuery().eq(NoteRecordSplit::getNoteRecordId, recordId).list();
                recordSplitProcessService.process(allTypeList, list);
                // 更新为1执行中
                throw new BusinessException("笔录解析任务未完成,请等待");
            }
        }
        // 这里进行查询
        return tripleInfoService.lambdaQuery().eq(TripleInfo::getRecordId, recordId).list();
    }


    @Override
    public void testExtractThreeInfo() {
//        -------------------------
//        DEMO_0:山东团队的版本,每次提取多个三元组,经过测试,提取质量较差,基本无法正常提取.需要改成每次提取一个,进行TEST_DEMO_1尝试
//        请分析以下内容中所有规定的三元组信息并补充完整，
//        要求：1.返回式：
//        {result：
//                   [
//            {startNodeType:'LawActor',entity:'',endNodeType:'FictionalOrgan',property:'',value:'冒充' },
//            {startNodeType:'LawActor',entity:'',endNodeType:'Seal',property:'',value:'伪造'},
//            {startNodeType:'LawActor',entity:'',endNodeType:'BusinessLicense',property:'',value:'伪造'}
//                   ]
//        }。
//        2.必须考虑上下文语境分析。
//        3.对话中不包含此类型三元组则返回{ "result": [] }。
//        例子1：办案警官问：你为了骗取更多的钱都做了哪些准备？裴金禄回答：我刚开始我就是自己想了一些关于骗钱的点子，后面为了更不容易让别人识破我为了更佳逼真，我就从网上随便搜了一家租赁公司，我就搜到了兰州胜利机械租赁有限公司，被讯问人：裴金禄我又想到了我管理的中铁北京局和中铁电气化局施工公司。我先是通过百度搜索了“办证”之后就在网页上面弹出了一个页面上面有一个QQ号，我就加上了。加上之后我就将我的要求给他说了，要求他给我刻两个假的公章，一个是兰州胜利机械租赁有限公司合同专用章，另一个是中铁北京局集团有限公司合同专用章。我还要求他给我伪造了一张兰州胜利机械租赁有限公司的营业执照。
//        回答：{"result":[{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"FictionalOrgan","property":"兰州胜利机械租赁有限公司","value":"冒充"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"Seal","property":"兰州胜利机械租赁有限公司合同专用章","value":"伪造"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"Seal","property":"中铁北京局集团有限公司合同专用章","value":"伪造"},{"startNodeType":"LawActor","entity":"裴金禄","endNodeType":"BusinessLicense","property":"兰州胜利机械租赁有限公司营业执照","value":"伪造"}]}。
//        例子2:办案警官问：你为什么要在中卫市沙坡头区签订这些合同？裴金禄回答：因为我在这边上班，我在中卫市沙坡头区签订合同更能有说服力，这样才能更好地骗到钱。回答：{ "result": [] }。

//        -------------------------
//        TEST_DEMO_1:结果:效果较差,容易出现混淆,会把例子中的实体给返回出来.如果提取不到,会把定义的关系返回出来,所以不行.PASS
//        请从
//        ---
//        办案警官问：你是如何想到这个方法骗钱的? 裴金禄回答：因为我是在中卫市沙坡头区中兰铁路上做监理人员，我这样说的话别人会更容易相信。
//        ---
//        这段话中按(主体类型:行为人姓名;关系:伪造;客体类型:合同名称)提取特定的三元组信息。
//        如果未提取到或关系不匹配或客体类型不匹配,则返回空的json即可。
//        如果可以提取到三元组关系,则返回格式为{"主体":"主体名称","关系":"关系名称","客体":"客体名称"}。
//        例如‘办案警官问:描述一下事情的经过。行为人小明答:我做了一份假的租车合同,然后骗小刚签了这个合同’，该示例存在给定的三元组关系.则结果为:{"主体":"小明","关系":"伪造","客体":"租车合同"}。
//        例如‘办案警官问:办案警官问：讲一下那个软件的名称？小明回答：现在叫畅聊。‘该示例不存在给定的三元组关系,则结果为:{}
//        请保证提取准确且完整，主体和客体要具体明确，不能是宽泛的概念。提取的客体要必须属于客体类型!
//        -------------------------
//        TEST_DEMO_2:结果:有一些结果可以,但总体正确率较低,问题:不能有效的对给定的三元组进行提取,会出现给定三元组关系之外的三元组.会出现文章中不存在的三元组或错误的三元组
//        三元组提取任务:从给定对话中根据给定实体类型和关系提取对应的三元组。
//        给定的头实体类型为"{headEntityType}";给定的尾实体类型为"{relation}",给定的关系为"{tailEntityType}"。
//        请仔细分析以下的文本内容，精准找出符合给定关系且头尾实体类型相符的三元组，并进行提取。如果没有识别给定的三元组关系，请返回json:{"result":[]}。
//        ---
//        为您提供一个示例供学习:
//        例如给定关系:给定三元组类型为:头实体类型:"行为人"关系:"伪造",尾实体类型:"合同"
//        例如QA:办案警官问:描述一下事情的经过。 行为人小明答:我做了一份假的购房合同。
//        本示例中应提取给定关系为"伪造"的三元组,则最终应提取的三元组为:{"result":[{"headEntity": {"type": "行为人","name":"小明"},"relation": "伪造","tailEntity": {"type": "合同","name": "假的购房合同"}}]}。
//        ---
//        需要分析提取的QA对如下：
//        {question}
//        {answer}
//        ---
//        在提取三元组时，请务必严格遵循以下要求：
//        1. 确保提取的三元组包括**给定的头实体类型**和**尾实体类型**。
//        2. 提取的三元组必须明确体现给定的**关系**。
//        3. 头实体和尾实体的名称必须在对话文本中找到。
//        4. 对于每个提取出的三元组，确保关系（{relation}）与实体类型完全匹配。
//        5. 如果文本中没有符合要求的三元组，请确保返回空结果，即 {"result": []}。
//        返回格式为必须为以下的json格式:
//        {"result":[{"headEntity": {"type": "给定头类型","name":"提取到的头实体内容1"},"relation": "给定关系","tailEntity": {"type": "给定尾类型","name": "提取到的尾实体内容1"}}]}
//        -------------------------
//        TEST_DEMO_3:尝试给定所有关系和头实体,只提取尾实体.结论:效果差,会把不相关的不符合的都提取出来,PASS.回退到DEMO2继续调整吧!
//        给定指标的存在性判断任务:给定一个指标,对给定QA进行判断是否符合,符合则为true,并把符合需实体类型的实体返回,不符合则为false
//        指标为:裴金禄是否{relation}{tailEntityType}(需提取实体类型为:{tailEntityType})
//        ---
//        为你提供一个实例进行学习:
//        示例QA:办案警官问:描述一下事情的经过。 行为人小明答:我做了一份假的购房合同。
//        示例指标:小明是否伪造合同(需提取实体为:合同)
//        示例结果:{"result":ture,"entity":["购房合同"]}
//        ---
//        需要分析的QA对如下：
//        {question}
//        {answer}
//        ---
//        请从需要分析的QA中进行判断提取!!!
//        在判断时，请务必仔细理解需要分析的QA文本的含义，确保提取的信息准确、合理。同时，尽量遵循常见的语义和逻辑规则，避免过度解读或不合理的关系推断。
//        返回格式为必须json格式,格式定义如下:
//        {"result":ture/false,"entity":["对应实体类型的实体"]}

        String prompt = """
                三元组提取任务:从给定对话中根据给定实体类型和关系提取对应的三元组。
                给定的头实体类型为"{headEntityType}";给定的尾实体类型为"{relation}",给定的关系为"{tailEntityType}"。
                请仔细分析以下的文本内容，精准找出符合给定关系且头尾实体类型相符的三元组，并进行提取。如果没有识别给定的三元组关系，请返回json:{"result":[]}。
                ---
                为您提供一个示例供学习:
                例如给定关系:给定三元组类型为:头实体类型:"行为人"关系:"伪造",尾实体类型:"合同"
                例如QA:办案警官问:描述一下事情的经过。 行为人小明答:我做了一份假的购房合同。
                本示例中应提取给定关系为"伪造"的三元组,则最终应提取的三元组为:{"result":[{"headEntity": {"type": "行为人","name":"小明"},"relation": "伪造","tailEntity": {"type": "合同","name": "假的购房合同"}}]}。
                ---
                需要分析提取的QA对如下：
                {question}
                {answer}
                ---
                在提取三元组时，请务必严格遵循以下要求：
                1. 确保提取的三元组包括**给定的头实体类型**和**尾实体类型**。
                2. 提取的三元组必须明确体现给定的**关系**。
                3. 头实体和尾实体的名称必须在对话文本中找到。
                4. 对于每个提取出的三元组，确保提取到的实体与给定实体类型完全匹配。
                5. 对于每个提取出的三元组，确保提取到的关系完全等于给定关系,如不等于,则提取错误。
                5. 如果文本中没有符合要求的三元组，请确保返回空结果，即 {"result": []}。
                返回格式为必须为以下的json格式:
                {"result":[{"headEntity": {"type": "给定头实体类型","name":"提取到的对应头实体内容1"},"relation": "给定关系","tailEntity": {"type": "给定实体尾类型","name": "提取到的对应尾实体内容1"}}]}
                """;
        // 抽取的客体名称一定属于给定的客体类型分类,且抽取的关系完全等于给定的关系。
        Prompt ask = new Prompt(new UserMessage(prompt));
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        log.info("开始分析:");
        ChatResponse call = chatClient.call(ask);
        stopWatch.stop();
        log.info("耗时:{}", stopWatch.getTotalTimeSeconds());
        String content = call.getResult().getOutput().getContent();
        log.info("分析的结果是:{}", content);
    }


    @Override
    public String addNeo4j(List<String> ids) {
        if (CollUtil.isEmpty(ids)) {
            return "成功插入0条信息";
        }
        List<TripleInfo> tripleInfos = tripleInfoService.listByIds(ids);
        int i = 0;
        for (TripleInfo tripleInfo : tripleInfos) {
            try {
                //开始节点
                String start = tripleInfo.getStartNode();
                // 首先看是否已经存在了,如果已经存在了,就不添加了
                CaseNode startNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getRecordId(), tripleInfo.getStartNodeType(), start, "1");
                if (startNode == null) {
                    startNode = new CaseNode(start, tripleInfo.getStartNodeType(), tripleInfo.getRecordSplitId(), tripleInfo.getRecordId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = neo4jService.save(startNode);
                    startNode.setId(save.getId());
                }
                //结束节点
                String end = tripleInfo.getEndNode();
                CaseNode endNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getRecordId(), tripleInfo.getEndNodeType(), end, "1");
                if (endNode == null) {
                    endNode = new CaseNode(end, tripleInfo.getEndNodeType(), tripleInfo.getRecordSplitId(), tripleInfo.getRecordId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = neo4jService.save(endNode);
                    endNode.setId(save.getId());
                }
                //关系
                Rel rel = new Rel(startNode.getId(), tripleInfo.getRelation(), endNode.getId());
                Rel relation = neo4jService.findRelation(rel);
                if (relation == null) {
                    relation = neo4jService.saveRelation(rel);
                }
                tripleInfo.setAddNeo4j("1");
                tripleInfo.setStartNodeGraphId(startNode.getId());
                tripleInfo.setRelGraphId(relation.getId());
                tripleInfo.setEndNodeGraphId(endNode.getId());
                boolean updateResult = tripleInfoService.updateById(tripleInfo);
                if (updateResult) {
                    i++;
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        // 删除的要进行移除
        Optional<TripleInfo> any = tripleInfos.stream().findAny();
        if (any.isPresent()) {
            TripleInfo anyTripleInfo = any.get();
            String recordId = anyTripleInfo.getRecordId();
            List<TripleInfo> existNeoList = tripleInfoService.lambdaQuery().eq(TripleInfo::getRecordId, recordId).eq(TripleInfo::getAddNeo4j, "1").list();
            Map<String, TripleInfo> existNeoMap = existNeoList.stream().collect(Collectors.toMap(TripleInfo::getId, Function.identity()));
            // 去掉已经添加的
            Set<String> existNeoIdSet = existNeoMap.keySet();
            ids.forEach(existNeoIdSet::remove);
            // 剩下的就是需要删除的
            for (String needDeleteId : existNeoIdSet) {
                TripleInfo tripleInfo = existNeoMap.get(needDeleteId);
                if (ObjectUtil.isNotEmpty(tripleInfo.getRelGraphId())) {
                    // 先移除关系
                    neo4jService.deleteRel(tripleInfo.getRelGraphId());
                } else {
                    break;
                }
                if (ObjectUtil.isNotEmpty(tripleInfo.getEndNodeGraphId())) {
                    // 再移除尾节点
                    neo4jService.delNode(tripleInfo.getEndNodeGraphId());
                } else {
                    break;
                }
                if (ObjectUtil.isNotEmpty(tripleInfo.getStartNodeGraphId())) {
                    // 尝试删除头节点(只有头节点不存在任何关系的时候才进行删除)
                    neo4jService.deleteNoRelationNode(tripleInfo.getStartNodeGraphId());
                } else {
                    break;
                }
                // 更新状态
                tripleInfoService.lambdaUpdate().set(TripleInfo::getAddNeo4j, "0")
                        .set(TripleInfo::getEndNodeGraphId, null)
                        .set(TripleInfo::getRelGraphId, null)
                        .set(TripleInfo::getStartNodeGraphId, null)
                        .eq(TripleInfo::getId, needDeleteId).update();
            }

        }

        return ("成功插入" + i + "条信息");
    }
}
