package com.supervision.police.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.police.domain.SystemMenu;
import com.supervision.police.dto.MenuDTO;
import com.supervision.police.service.SystemMenuService;
import com.supervision.police.mapper.SystemMenuMapper;
import org.springframework.stereotype.Service;

import java.util.*;

/**
* @author Administrator
* @description 针对表【system_menu(菜单表)】的数据库操作Service实现
* @createDate 2024-07-31 11:02:43
*/
@Service
public class SystemMenuServiceImpl extends ServiceImpl<SystemMenuMapper, SystemMenu>
    implements SystemMenuService{

    @Override
    public List<MenuDTO> treeMenuList() {
        List<MenuDTO> menuDTOS = listMenu();
        return treeMenuList(menuDTOS);
    }

    @Override
    public List<MenuDTO> treeMenuList(List<MenuDTO> menuDTOList) {

        if (CollUtil.isEmpty(menuDTOList)){
            return new ArrayList<>();
        }

        for (MenuDTO menuDTO : menuDTOList) {
            List<MenuDTO> children = menuDTOList.stream()
                    .filter(item -> menuDTO.getId().equals(item.getParentId())).toList();
            menuDTO.setChildren(children);
        }

        return menuDTOList.stream().filter(item -> StrUtil.isEmpty(item.getParentId())).toList();
    }

    @Override
    public List<MenuDTO> listMenu() {
        return super.list().stream().map(MenuDTO::new).toList();
    }

    @Override
    public List<MenuDTO> filterMenuBySelect(List<MenuDTO> menuDTOList, List<String> selectMenuIdList) {
        if (CollUtil.isEmpty(menuDTOList) || CollUtil.isEmpty(selectMenuIdList)){
            return new ArrayList<>(1);
        }

        List<MenuDTO> seekMenuDTOList = new ArrayList<>();
        for (String menuId : selectMenuIdList) {
            for (MenuDTO menuDTO : menuDTOList) {
                List<MenuDTO> menuDTOS = menuDTO.seekMenu(menuId);
                if (CollUtil.isNotEmpty(menuDTOS)){
                    seekMenuDTOList.addAll(menuDTOS);
                }
            }
        }

        return seekMenuDTOList;
    }



}




