package com.supervision.utils;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.supervision.police.dto.user.UserInfoDTO;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

public class UserUtil {

    public static UserInfoDTO getUser() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        Assert.notNull(requestAttributes, "未获取到用户信息");
        HttpServletRequest request = requestAttributes.getRequest();

        String token = request.getHeader("token");
        if (StrUtil.isBlank(token)) {
            token = (String) request.getAttribute("token");

        }
        Assert.notEmpty(token, "未获取到用户token信息");
        JWT jwt = JWTUtil.parseToken(token);
        JSONObject claimsJson = jwt.getPayload().getClaimsJson();
        UserInfoDTO bean = JSONUtil.toBean(claimsJson.toString(), UserInfoDTO.class);
        Assert.notNull(bean, "未获取到用户信息");
        return bean;
    }


    /**
     * 验证密码
     * @param password 明文密码
     * @param userPd 密文签名
     * @return
     */
    public static boolean checkUserPassword(String password, String userPd) {
        return SM2Util.verify(password, userPd);
    }

    /**
     * 密码加密签名
     * @param password 密码
     * @return
     */
    public static String signPassword(String password){
        return SM2Util.sign(password);
    }

}
