package com.supervision.demo;

import com.alibaba.druid.sql.ast.SQLStatement;
import com.alibaba.druid.sql.ast.statement.*;
import com.alibaba.druid.sql.dialect.mysql.parser.MySqlStatementParser;
import com.alibaba.druid.sql.dialect.mysql.visitor.MySqlSchemaStatVisitor;
import com.alibaba.druid.stat.TableStat;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class DruidTableExtractor {
    public static void main(String[] args) {
        String sql = "drop u.name, o.id FROM users u left JOIN orders o ON u.id = o.user_id left join dept d on u.id = d.id WHERE u.id = 1 and u.id in ( select id from people where name = 'dd' )";


        boolean b = validateSQL(sql);
        System.out.println("SQL 是否合法: " + b);


        String s = detectSQLType(sql);
        System.out.println("SQL 类型: " + s);

        List<String> tableNames = extractTableNames(sql);
        System.out.println("涉及到的表: " + tableNames);
    }

    public static boolean validateSQL(String sql) {
        try {
            // 解析 SQL
            MySqlStatementParser parser = new MySqlStatementParser(sql);
            SQLStatement statement = parser.parseStatement();
            return true;  // 解析成功，SQL 语句合法
        } catch (Exception e) {
            // 解析异常，SQL 语句不合法
            System.out.println("SQL 解析错误: " + e.getMessage());
            return false;
        }
    }

    public static String detectSQLType(String sql) {
        try {
            // 解析 SQL
            MySqlStatementParser parser = new MySqlStatementParser(sql);
            SQLStatement statement = parser.parseStatement();

            // 判断 SQL 类型
            if (statement instanceof SQLSelectStatement) {
                return "SELECT";
            } else if (statement instanceof SQLInsertStatement) {
                return "INSERT";
            } else if (statement instanceof SQLUpdateStatement) {
                return "UPDATE";
            } else if (statement instanceof SQLDeleteStatement) {
                return "DELETE";
            } else {
                return "UNKNOWN";
            }
        } catch (Exception e) {
            System.out.println("SQL 解析错误: " + e.getMessage());
            return "ERROR";
        }
    }

    public static List<String> extractTableNames(String sql) {
        // 解析 SQL
        MySqlStatementParser parser = new MySqlStatementParser(sql);
        SQLStatement statement = parser.parseStatement();

        // 访问者模式提取表名
        MySqlSchemaStatVisitor visitor = new MySqlSchemaStatVisitor();
        statement.accept(visitor);

        Set<TableStat.Name> tableStatNames = visitor.getTables().keySet();
        List<String> tableNames = new ArrayList<>();

        for (TableStat.Name name : tableStatNames) {
            tableNames.add(name.getName());
        }

        return tableNames;
    }
}

